package com.sztzjy.resource_center.controller;

import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.*;
import com.sztzjy.resource_center.mapper.SysCaseQuestionMapper;
import com.sztzjy.resource_center.mapper.SysCaseQuestionStepMapper;
import com.sztzjy.resource_center.mapper.SysTopicAndCourseMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author xcj
 * @Date 2024/6/26
 */
@RestController
@Api(tags = "案例题步骤")
@RequestMapping("api/sys/case/step")
public class CaseStepController {
    @Autowired
    private SysCaseQuestionStepMapper caseQuestionStepMapper;
    @Autowired
    private SysTopicAndCourseMapper topicAndCourseMapper;
    @Autowired
    private SysCaseQuestionMapper sysCaseQuestionMapper;


    @AnonymousAccess
    @ApiOperation("案例步骤展示")
    @PostMapping("selectCaseStep")
    public ResultEntity<List<SysCaseQuestionStepWithBLOBs>> selectCaseStep(@ApiParam("案例题ID") @RequestParam String caseId) {
        SysCaseQuestionStepExample example = new SysCaseQuestionStepExample();
        example.createCriteria().andCaseIdEqualTo(caseId);
        example.setOrderByClause("sort ASC");
        List<SysCaseQuestionStepWithBLOBs> stepList = caseQuestionStepMapper.selectByExampleWithBLOBs(example);
        return new ResultEntity<>(stepList);
    }

    @GetMapping("selectCaseStepDetails")
    @ApiOperation("案例题步骤内容查看")
    public ResultEntity<SysCaseQuestionStep> selectCaseStepDetails(@ApiParam("案例题步骤ID") @RequestParam String caseStepId) {
        SysCaseQuestionStep sysCaseQuestionStep = caseQuestionStepMapper.selectByPrimaryKey(caseStepId);
        return new ResultEntity<>(HttpStatus.OK, "案例题步骤内容查看成功", sysCaseQuestionStep);
    }


    @AnonymousAccess
    @ApiOperation("案例题步骤编辑")
    @PostMapping("updateCase")
    public ResultEntity<HttpStatus> updateCase(@RequestBody SysCaseQuestionStepWithBLOBs caseQuestionStepWithBLOBs,
                                               @ApiParam("谁调的请求，传管理员/学校ID") @RequestParam String source) {
        List<SysTopicAndCourse> sysTopicAndCourses = getSysTopicAndCourses(caseQuestionStepWithBLOBs.getCaseId());
        if (!sysTopicAndCourses.isEmpty()) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "该案例题正在被使用!");
        } else {
            //todo 管理员任意修改，老师只能改自己的
            SysCaseQuestion sysCaseQuestion = sysCaseQuestionMapper.selectByPrimaryKey(caseQuestionStepWithBLOBs.getCaseId());
            if ("管理员".equals(source) || source.equals(sysCaseQuestion.getSource())) {
                caseQuestionStepMapper.updateByPrimaryKey(caseQuestionStepWithBLOBs);
                return new ResultEntity<>(HttpStatus.OK, "编辑成功!");
            } else {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "权限不足!");
            }
        }
    }


    @AnonymousAccess
    @ApiOperation("案例题步骤删除")
    @PostMapping("deleteCase")
    public ResultEntity<HttpStatus> deleteCase(@RequestBody SysCaseQuestionStepWithBLOBs sysCaseQuestionStep,
                                               @ApiParam("哪个用的掉的请求，传管理员/学校ID") @RequestParam String source) {
        List<SysTopicAndCourse> sysTopicAndCourses = getSysTopicAndCourses(sysCaseQuestionStep.getCaseId());
        if (!sysTopicAndCourses.isEmpty()) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "无法删除！该案例题正在被使用!");
        } else {
            //todo 管理员任意删除 老师只能删除自己上传的
            SysCaseQuestion sysCaseQuestion = sysCaseQuestionMapper.selectByPrimaryKey(sysCaseQuestionStep.getCaseId());
            if ("管理员".equals(source) || source.equals(sysCaseQuestion.getSource())) {
                caseQuestionStepMapper.deleteByPrimaryKey(sysCaseQuestionStep.getCaseStepId());
                return new ResultEntity<>(HttpStatus.OK, "删除成功!");
            } else {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "权限不足!");
            }
        }
    }

    private List<SysTopicAndCourse> getSysTopicAndCourses(String caseId) {
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andTopicIdEqualTo(caseId).andTopicTypeEqualTo("1");
        return topicAndCourseMapper.selectByExample(example);
    }


}
