package com.sztzjy.resource_center.controller;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.SysCaseQuestionExample;
import com.sztzjy.resource_center.entity.SysResource;
import com.sztzjy.resource_center.entity.SysResourceAndCourse;
import com.sztzjy.resource_center.entity.SysResourceAndCourseExample;
import com.sztzjy.resource_center.entity.dto.SysResourceDto;
import com.sztzjy.resource_center.mapper.SysCaseQuestionMapper;
import com.sztzjy.resource_center.mapper.SysResourceAndCourseMapper;
import com.sztzjy.resource_center.mapper.SysResourceMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import com.sztzjy.resource_center.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;
import java.util.List;

/**
 * @Author xcj
 * @Date 2024/6/13
 */

@RestController
@Api(tags = "课程配置")
@RequestMapping("api/sys/course/config")
public class CourseConfigController {
    @Autowired
    private SysResourceMapper sysResourceMapper;
    @Autowired
    private SysResourceAndCourseMapper sysResourceAndCourseMapper;
    @Autowired
    private SysCaseQuestionMapper sysCaseQuestionMapper;
    @Autowired
    private IFileUtil fileUtil;

    @AnonymousAccess
    @ApiOperation("本地上传资源")
    @PostMapping("uploadLocal")
    public ResultEntity<String> uploadLocal(@ModelAttribute SysResourceDto dto) {
        if (dto.getFile() == null) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请上传文件！");
        }
        if (dto.getPictureUrl() == null) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请上传封面图片！");
        }
        //取到文件后缀名
        String fileName = dto.getResourceName();
        int lastIndex = fileName.lastIndexOf(".");
        String extension = fileName.substring(lastIndex);

        //图片名字
        MultipartFile pictureUrl = dto.getPictureUrl();
        String pictureName = pictureUrl.getName();

        SysResource sysResource = new SysResource();
        //判断文件type   //todo 格式可能不够全
        if (extension.equals(".pdf") || extension.equals(".ppt") || extension.equals(".pptx")
                || extension.equals(".xlsx") || extension.equals(".xls")
                || extension.equals(".doc") || extension.equals(".docx")) {
            sysResource.setResourceType("课件");
        } else if (extension.equals(".mp4") || extension.equals(".wmv") || extension.equals(".avi")) {
            sysResource.setResourceType("视频");
        } else {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "文件类型不支持！");
        }
        String url = fileUtil.upload(dto.getFile());
        String uuid = IdUtil.randomUUID();
        sysResource.setResourceId(uuid);
        sysResource.setPictureUrl(pictureName);
        sysResource.setUrl(url);
        sysResource.setSource(dto.getSource());  //管理员或者 xx学校
        sysResource.setResourceName(dto.getResourceName());
        sysResource.setStatus(1);
        sysResource.setCreateTime(new Date());  //todo 测试修改资源之后 新增时间会不会变化，有变化就错了
        sysResourceMapper.insert(sysResource);

        //绑定资源的对应关系
        SysResourceAndCourse sysResourceAndCourse = new SysResourceAndCourse();
        sysResourceAndCourse.setId(IdUtil.randomUUID());
        sysResourceAndCourse.setOneId(dto.getOneId());
        sysResourceAndCourse.setTwoId(dto.getTwoId());
        sysResourceAndCourse.setThreeId(dto.getThreeId());
        sysResourceAndCourse.setOneName(dto.getOneName());
        sysResourceAndCourse.setTwoName(dto.getTwoName());
        sysResourceAndCourse.setThreeName(dto.getThreeName());
        sysResourceAndCourse.setResourceId(uuid);  //资源ID
        sysResourceAndCourseMapper.insert(sysResourceAndCourse);
        return new ResultEntity<>(HttpStatus.OK, "上传成功！");
    }


    @AnonymousAccess
    @ApiOperation("资源库上传多个资源") //资源已有，添加绑定关系即可
    @PostMapping("uploadByResources")
    public ResultEntity<String> uploadByResources(@RequestBody List<SysResourceAndCourse> resourceAndCourses) {
        for (SysResourceAndCourse resourceAndCourse : resourceAndCourses) {
            if (StringUtils.isBlank(resourceAndCourse.getOneId())
                    || StringUtils.isBlank(resourceAndCourse.getTwoId())
                    || StringUtils.isBlank(resourceAndCourse.getThreeId())) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "缺少必要的ID");
            } else {
                resourceAndCourse.setId(IdUtil.randomUUID());
            }
        }
        try {
            sysResourceAndCourseMapper.batchInsert(resourceAndCourses);
        } catch (Exception e) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "删除失败，请联系管理员！");
        }
        return new ResultEntity<>(HttpStatus.OK, "添加成功！");
    }


    @AnonymousAccess
    @ApiOperation("课程配置/资源列表展示，无重复")
    @PostMapping("selectResourceByIDAndSource")
    public ResultEntity<PageInfo<SysResourceDto>> selectResource(@ApiParam("课程ID") @RequestParam(required = false) String oneID,
                                                                 @ApiParam("章ID") @RequestParam(required = false) String twoID,
                                                                 @ApiParam("节ID") @RequestParam(required = false) String threeID,
                                                                 @RequestParam(required = false) String source, //子系统调用时传
                                                                 @RequestParam Integer index,
                                                                 @RequestParam Integer size) {
        PageHelper.startPage(index, size);
        List<SysResourceDto> sysResourceDtos = sysResourceMapper.selectResource(oneID, twoID, threeID);
        PageInfo<SysResourceDto> pageInfo = new PageInfo(sysResourceDtos);
        return new ResultEntity<PageInfo<SysResourceDto>>(pageInfo);
    }


    @AnonymousAccess
    @ApiOperation("课程配置删除资源")  //只删除绑定关系
    @PostMapping("deleteResource")
    public ResultEntity<String> deleteResource(@RequestParam String id,
                                               @RequestParam(required = false) String oneId,
                                               @RequestParam(required = false) String twoId,
                                               @RequestParam(required = false) String threeId) {
        SysResourceAndCourseExample example = new SysResourceAndCourseExample();
        SysResourceAndCourseExample.Criteria criteria = example.createCriteria().andResourceIdEqualTo(id);
        if (StringUtils.isNotBlank(oneId)) {
            criteria.andOneIdEqualTo(oneId);
        }
        if (StringUtils.isNotBlank(twoId)) {
            criteria.andTwoIdEqualTo(twoId);
        }
        if (StringUtils.isNotBlank(threeId)) {
            criteria.andThreeIdEqualTo(threeId);
        }
        try {
            sysResourceAndCourseMapper.deleteByExample(example);
        } catch (Exception e) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "删除失败，请联系管理员！");
        }
        return new ResultEntity<>(HttpStatus.OK, "删除成功！");
    }

}