package com.sztzjy.resource_center.controller;


import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.*;
import com.sztzjy.resource_center.mapper.SysOneCatalogMapper;
import com.sztzjy.resource_center.mapper.SysResourceAndCourseMapper;
import com.sztzjy.resource_center.mapper.SysThreeCatalogMapper;
import com.sztzjy.resource_center.mapper.SysTwoCatalogMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @Author xcj
 * @Date 2024/6/13
 */
@RestController
@Api(tags = "课程管理")
@RequestMapping("api/sys/course/manage")
public class CourseManageController {

    @Autowired
    private SysOneCatalogMapper sysOneCatalogMapper;
    @Autowired
    private SysTwoCatalogMapper sysTwoCatalogMapper;
    @Autowired
    private SysThreeCatalogMapper sysThreeCatalogMapper;
    @Autowired
    private SysResourceAndCourseMapper sysResourceAndCourseMapper;

    @AnonymousAccess
    @ApiOperation("新增课程")
    @PostMapping("insertSysOneCatalog")
    public ResultEntity<String> getIndexCourse(@ApiParam("课程链接必填,课程简介选填") @RequestBody SysOneCatalog sysOneCatalog) {
        if (StringUtils.isBlank(sysOneCatalog.getCourseUrl())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请输入课程链接！");
        }
        if (StringUtils.isBlank(sysOneCatalog.getOneName())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请输入课程名字！");
        }
        SysOneCatalogExample sysOneCatalogExample = new SysOneCatalogExample();
        sysOneCatalogExample.createCriteria().andOneNameEqualTo(sysOneCatalog.getOneName());
        List<SysOneCatalog> sysOneCatalogs = sysOneCatalogMapper.selectByExample(sysOneCatalogExample);
        if (!sysOneCatalogs.isEmpty()) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "该课程已存在，请重新输入！");
        } else {
            sysOneCatalog.setOneId(IdUtil.randomUUID());
            sysOneCatalog.setCreateTime(new Date());
            sysOneCatalog.setStatus(1);
            sysOneCatalogMapper.insert(sysOneCatalog);
            return new ResultEntity<>(HttpStatus.OK, "新增成功");
        }
    }

    @AnonymousAccess
    @ApiOperation("编辑课程/启用停用")
    @PostMapping("updateSysOneCatalog")
    public ResultEntity<String> updateSysOneCatalog(@ApiParam("课程链接必填,课程简介选填") @RequestBody SysOneCatalog sysOneCatalog) {
        sysOneCatalogMapper.updateByPrimaryKey(sysOneCatalog);
        return new ResultEntity<>(HttpStatus.OK, "编辑成功");
    }


    @AnonymousAccess
    @ApiOperation("课程下拉框/资源标签管理展示")
    @PostMapping("getAllOneCatalogListByDropDown")
    public ResultEntity<List<SysOneCatalog>> getAllOneCatalogListByDropDown() {
        SysOneCatalogExample example = new SysOneCatalogExample();
        List<SysOneCatalog> sysOneCatalogs = sysOneCatalogMapper.selectByExample(example);
        return new ResultEntity<>(sysOneCatalogs);
    }


    @AnonymousAccess
    @ApiOperation("删除课程")
    @PostMapping("deleteOneCatalog")
    public ResultEntity<String> deleteOneCatalog(@RequestParam String oneId) {
        sysOneCatalogMapper.deleteByPrimaryKey(oneId);
        //删除课程资源的绑定关系
        SysResourceAndCourseExample example = new SysResourceAndCourseExample();
        example.createCriteria().andOneIdEqualTo(oneId);
        sysResourceAndCourseMapper.deleteByExample(example);
        return new ResultEntity<>(HttpStatus.OK, "删除成功");
    }

    @AnonymousAccess
    @ApiOperation("章下拉框/资源标签管理展示")
    @PostMapping("getAllTwoCatalogListByDropDown")
    public ResultEntity<List<SysTwoCatalog>> getAllTwoCatalogList(@RequestParam String oneId) {
        SysTwoCatalogExample example = new SysTwoCatalogExample();
        example.createCriteria().andOneIdEqualTo(oneId);
        List<SysTwoCatalog> sysTwoCatalogs = sysTwoCatalogMapper.selectByExample(example);
        return new ResultEntity<>(sysTwoCatalogs);
    }


    @AnonymousAccess
    @ApiOperation("节下拉框/资源标签管理展示")
    @PostMapping("getAllThreeCatalogListByDropDown")
    public ResultEntity<List<SysThreeCatalog>> getAllThreeCatalogList(@RequestParam String oneId, @RequestParam String twoId) {
        SysThreeCatalogExample example = new SysThreeCatalogExample();
        example.createCriteria().andOntIdEqualTo(oneId).andTwoIdEqualTo(twoId);
        List<SysThreeCatalog> sysThreeCatalogs = sysThreeCatalogMapper.selectByExample(example);
        return new ResultEntity<>(sysThreeCatalogs);
    }


    @AnonymousAccess
    @ApiOperation("课程管理页面展示")
    @PostMapping("getOneCatalogList")
    public ResultEntity<PageInfo<SysOneCatalog>> getOneCatalogList(@RequestParam Integer index,
                                                                   @RequestParam Integer size) {
        PageHelper.startPage(index, size);
        SysOneCatalogExample example = new SysOneCatalogExample();
        example.setOrderByClause("sort ASC");
        List<SysOneCatalog> sysOneCatalogs = sysOneCatalogMapper.selectByExample(example);
        PageInfo<SysOneCatalog> pageInfo = new PageInfo(sysOneCatalogs);
        return new ResultEntity<PageInfo<SysOneCatalog>>(pageInfo);
    }

}
