package com.sztzjy.resource_center.controller;

import cn.hutool.core.util.IdUtil;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.config.security.JwtUser;
import com.sztzjy.resource_center.config.security.TokenProvider;
import com.sztzjy.resource_center.entity.SysOneCatalog;
import com.sztzjy.resource_center.entity.SysThreeCatalog;
import com.sztzjy.resource_center.entity.SysTwoCatalog;
import com.sztzjy.resource_center.mapper.SysOneCatalogMapper;
import com.sztzjy.resource_center.mapper.SysThreeCatalogMapper;
import com.sztzjy.resource_center.mapper.SysTwoCatalogMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author xcj
 * @Date 2024/6/18
 */
@RestController
@Api(tags = "资源标签管理") //展示接口同课程管理下拉框
@RequestMapping("api/sys/resource/manage")
public class CourseTagManageController {
    @Autowired
    private SysOneCatalogMapper sysOneCatalogMapper;
    @Autowired
    private SysTwoCatalogMapper sysTwoCatalogMapper;
    @Autowired
    private SysThreeCatalogMapper sysThreeCatalogMapper;
    @Resource
    private AuthenticationManagerBuilder authenticationManagerBuilder;

    @AnonymousAccess
    @ApiOperation("添加二级目录")
    @PostMapping("insertSysTwoCatalog")
    public ResultEntity<String> insertSysTwoCatalog(@RequestBody SysTwoCatalog sysTwoCatalog) {
        if (StringUtils.isBlank(sysTwoCatalog.getOneId())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "缺少一级目录ID！");
        }
        if (StringUtils.isBlank(sysTwoCatalog.getTwoName())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请输入目录名称！");
        }
        sysTwoCatalog.setTwoId(IdUtil.randomUUID());
        sysTwoCatalog.setCreateTime(new Date());
        sysTwoCatalogMapper.insert(sysTwoCatalog);
        return new ResultEntity<>(HttpStatus.OK, "新增成功");
    }

    @AnonymousAccess
    @ApiOperation("添加三级目录")
    @PostMapping("insertSysThreeCatalog")
    public ResultEntity<String> insertSysThreeCatalog(@RequestBody SysThreeCatalog sysThreeCatalog) {
        if (StringUtils.isBlank(sysThreeCatalog.getOntId())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "缺少一级目录ID！");
        }
        if (StringUtils.isBlank(sysThreeCatalog.getTwoId())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "缺少二级目录ID！");
        }
        if (StringUtils.isBlank(sysThreeCatalog.getThreeName())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请输入目录名称！");
        }
        sysThreeCatalog.setThreeId(IdUtil.randomUUID());
        sysThreeCatalog.setCreateTime(new Date());
        sysThreeCatalogMapper.insert(sysThreeCatalog);
        return new ResultEntity<>(HttpStatus.OK, "新增成功");
    }


    @AnonymousAccess
    @ApiOperation("修改二级或三级目录名称")
    @PostMapping("updateTwoAndThreeCatalog")
    public ResultEntity<String> updateTwoAndThreeCatalog(@RequestParam String name,
                                                         @ApiParam("二级目录传2/三级传3") @RequestParam int type,
                                                         @RequestParam String id) {
        if (type == 2) {
            SysTwoCatalog sysTwoCatalog = sysTwoCatalogMapper.selectByPrimaryKey(id);
            sysTwoCatalog.setTwoName(name);
            sysTwoCatalogMapper.updateByPrimaryKey(sysTwoCatalog);
        } else {
            SysThreeCatalog sysThreeCatalog = sysThreeCatalogMapper.selectByPrimaryKey(id);
            sysThreeCatalog.setThreeName(name);
            sysThreeCatalogMapper.updateByPrimaryKey(sysThreeCatalog);
        }
        return new ResultEntity<>(HttpStatus.OK, "修改成功");
    }


    @AnonymousAccess
    @ApiOperation("删除二级或三级目录名称")
    @PostMapping("deleteTwoAndThreeCatalog")
    public ResultEntity<String> deleteTwoAndThreeCatalog(@ApiParam("二级目录传2/三级传3") @RequestParam int type,
                                                         @RequestParam String id) {
        if (type == 2) {
            sysTwoCatalogMapper.deleteByPrimaryKey(id);
        } else {
            sysThreeCatalogMapper.deleteByPrimaryKey(id);
        }
        return new ResultEntity<>(HttpStatus.OK, "删除成功");
    }

//    @AnonymousAccess
//    @ApiOperation("排序")
//    @PostMapping("setRank")  //修改成LIST
//    public ResultEntity<String> setRank(@ApiParam("一级传1/二级传2/三级传3") @RequestParam int type,
//                                        @RequestParam String id,
//                                        @RequestParam int rank) {
//        if (type == 1) {
//            SysOneCatalog sysOneCatalog = sysOneCatalogMapper.selectByPrimaryKey(id);
//            sysOneCatalog.setSort(rank);
//            sysOneCatalogMapper.updateByPrimaryKey(sysOneCatalog);
//            return new ResultEntity<>(HttpStatus.OK, "排序成功");
//        } else if (type == 2) {
//            SysTwoCatalog sysTwoCatalog = sysTwoCatalogMapper.selectByPrimaryKey(id);
//            sysTwoCatalog.setSort(rank);
//            sysTwoCatalogMapper.updateByPrimaryKey(sysTwoCatalog);
//            return new ResultEntity<>(HttpStatus.OK, "排序成功");
//        } else {
//            SysThreeCatalog sysThreeCatalog = sysThreeCatalogMapper.selectByPrimaryKey(id);
//            sysThreeCatalog.setSort(rank);
//            sysThreeCatalogMapper.updateByPrimaryKey(sysThreeCatalog);
//            return new ResultEntity<>(HttpStatus.OK, "排序成功");
//        }
//    }


    @AnonymousAccess
    @ApiOperation("一级目录重新排序")
    @PostMapping("setOneCatalogueRank")
    public ResultEntity setCatalogueRank(@ApiParam("需要ID和sort") @RequestBody List<SysOneCatalog> sysOneCatalogs) {
        for (SysOneCatalog sysOneCatalog : sysOneCatalogs) {
            if (StringUtils.isBlank(sysOneCatalog.getOneId()) || sysOneCatalog.getSort() == null) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "更新失败，缺少必须的数据！");
            }
        }
        sysOneCatalogMapper.updateByBatch(sysOneCatalogs);
        return null;
    }


    @AnonymousAccess
    @ApiOperation("二级目录重新排序")
    @PostMapping("setTwoCatalogueRank")
    public ResultEntity setTwoCatalogueRank(@ApiParam("一级ID（可为空），二级ID和sort必传")@RequestBody List<SysTwoCatalog> sysTwoCatalogs) {
        for (SysTwoCatalog sysTwoCatalog : sysTwoCatalogs) {
            if (StringUtils.isBlank(sysTwoCatalog.getTwoId())
                    || sysTwoCatalog.getSort() == null) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "更新失败，缺少必须的数据！");
            }
        }
        sysTwoCatalogMapper.updateByBatch(sysTwoCatalogs);
        return null;
    }


    @AnonymousAccess
    @ApiOperation("三级目录重新排序")
    @PostMapping("setThreeCatalogueRank")
    public ResultEntity setThreeCatalogueRank(@ApiParam("需要一级（可为空）二级（可为空），三级ID和sort必传")@RequestBody List<SysThreeCatalog> sysThreeCatalogs) {
        for (SysThreeCatalog sysThreeCatalog : sysThreeCatalogs) {
            if (StringUtils.isBlank(sysThreeCatalog.getThreeId()) || sysThreeCatalog.getSort() == null) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "更新失败，缺少必须的数据！");
            }
        }
        sysThreeCatalogMapper.updateByBatch(sysThreeCatalogs);
        return null;
    }


    @PostMapping("/login")
    @ApiOperation("登录接口")
    @AnonymousAccess

    public ResultEntity login(@RequestParam String password,
                              @RequestParam String userName) {
        if (StringUtils.isBlank(password)
                || !userName.equals("admin")
                || StringUtils.isBlank(userName)
                || !password.equals("tianze@admin@123")) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "账号或密码不正确!");
        } else {
            Map<String, Object> map = new HashMap();
            map.put("username", userName);
            JwtUser jwtUser = new JwtUser();
            jwtUser.setUsername(userName);
            jwtUser.setPassword(password);
            String token = TokenProvider.createToken(jwtUser);
            map.put("token", token);
            return new ResultEntity(map);
        }
    }
}
