package com.sztzjy.resource_center.controller;

import cn.hutool.core.util.IdUtil;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.SysKnowledgeAndCourse;
import com.sztzjy.resource_center.entity.SysKnowledgeAndCourseExample;
import com.sztzjy.resource_center.entity.SysKnowledgeSummary;
import com.sztzjy.resource_center.entity.SysKnowledgeSummaryExample;
import com.sztzjy.resource_center.mapper.SysKnowledgeAndCourseMapper;
import com.sztzjy.resource_center.mapper.SysKnowledgeSummaryMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author xcj
 * @Date 2024/7/11
 */
@RestController
@Api(tags = "知识概要页面")
@RequestMapping("api/sys/knowledgeSummary")
public class KnowledgeSummaryController {

    @Autowired
    private SysKnowledgeSummaryMapper knowledgeSummaryMapper;
    @Autowired
    private SysKnowledgeAndCourseMapper knowledgeAndCourseMapper;

    @AnonymousAccess
    @ApiOperation("知识概要新增或修改")
    @PostMapping("addOrUpdate")
    public ResultEntity<String> addOrUpdate(@ApiParam("没传ID为新增，传了为修改，新增传source字段") @RequestBody SysKnowledgeSummary sysKnowledgeSummary) {
        if (StringUtils.isBlank(sysKnowledgeSummary.getContent())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "内容不能为空！");
        }
        //不为空修改
        SysKnowledgeSummaryExample example = new SysKnowledgeSummaryExample();
        SysKnowledgeSummaryExample.Criteria criteria = example.createCriteria();
        criteria.andOneTagIdEqualTo(sysKnowledgeSummary.getOneTagId()).andThreeTagIdEqualTo(sysKnowledgeSummary.getThreeTagId()).andTwoTagIdEqualTo(sysKnowledgeSummary.getTwoTagId());
        List<SysKnowledgeSummary> sysKnowledgeSummaries = knowledgeSummaryMapper.selectByExampleWithBLOBs(example);
        if (sysKnowledgeSummaries != null && !sysKnowledgeSummaries.isEmpty()) {
            knowledgeSummaryMapper.updateByPrimaryKeyWithBLOBs(sysKnowledgeSummary);
            return new ResultEntity<>(HttpStatus.OK, "修改成功!");
        }

        sysKnowledgeSummary.setKnowledgeSummaryId(IdUtil.randomUUID());
        knowledgeSummaryMapper.insert(sysKnowledgeSummary);

        SysKnowledgeAndCourse sysKnowledgeAndCourse = new SysKnowledgeAndCourse();
        sysKnowledgeAndCourse.setId(IdUtil.randomUUID());
        if (StringUtils.isNotBlank(sysKnowledgeSummary.getOneTagId())) {
            sysKnowledgeAndCourse.setOneId(sysKnowledgeSummary.getOneTagId());
            sysKnowledgeAndCourse.setOneName(sysKnowledgeSummary.getOneTagName());
        }
        if (StringUtils.isNotBlank(sysKnowledgeSummary.getTwoTagId())) {
            sysKnowledgeAndCourse.setTwoId(sysKnowledgeSummary.getTwoTagId());
            sysKnowledgeAndCourse.setTwoName(sysKnowledgeSummary.getTwoTagName());
        }
        if (StringUtils.isNotBlank(sysKnowledgeSummary.getThreeTagId())) {
            sysKnowledgeAndCourse.setThreeId(sysKnowledgeSummary.getThreeTagId());
            sysKnowledgeAndCourse.setThreeName(sysKnowledgeSummary.getThreeTagName());
        }
        knowledgeAndCourseMapper.insert(sysKnowledgeAndCourse);
        return new ResultEntity<>(HttpStatus.OK, "保存成功!");
    }

    @AnonymousAccess
    @ApiOperation("知识概要删除数据")
    @PostMapping("deleteKnowledgeSummary")
    public ResultEntity<String> deleteKnowledgeSummary(@RequestParam String id) {
        SysKnowledgeAndCourseExample example = new SysKnowledgeAndCourseExample();
        example.createCriteria().andKnowledgeSummaryIdEqualTo(id);
        List<SysKnowledgeAndCourse> sysKnowledgeAndCourses = knowledgeAndCourseMapper.selectByExample(example);
        if (sysKnowledgeAndCourses != null && !sysKnowledgeAndCourses.isEmpty()) {
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "该知识概要正在被使用，请先删除绑定关系!");
        } else {
            knowledgeSummaryMapper.deleteByPrimaryKey(id);
            return new ResultEntity<>(HttpStatus.OK, "删除成功！");
        }
    }

    @AnonymousAccess
    @ApiOperation("知识概要删除绑定关系")
    @PostMapping("deleteKnowledgeSummaryBind")
    public ResultEntity<String> deleteKnowledgeSummaryBind(@RequestParam String id,
                                                           @RequestParam(required = false) String oneId,
                                                           @RequestParam(required = false) String twoId,
                                                           @RequestParam(required = false) String threeId) {
        SysKnowledgeAndCourseExample example = new SysKnowledgeAndCourseExample();
        SysKnowledgeAndCourseExample.Criteria criteria = example.createCriteria();
        criteria.andKnowledgeSummaryIdEqualTo(id);
        if (StringUtils.isNotBlank(oneId)) {
            criteria.andOneIdEqualTo(oneId);
        }
        if (StringUtils.isNotBlank(twoId)) {
            criteria.andTwoIdEqualTo(twoId);
        }
        if (StringUtils.isNotBlank(threeId)) {
            criteria.andThreeIdEqualTo(threeId);
        }
        List<SysKnowledgeAndCourse> sysKnowledgeAndCourses = knowledgeAndCourseMapper.selectByExample(example);
        if (sysKnowledgeAndCourses == null || sysKnowledgeAndCourses.isEmpty()) {
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "该知识概要暂无绑定关系!");
        } else {
            knowledgeAndCourseMapper.deleteByExample(example);
            return new ResultEntity<>(HttpStatus.OK, "删除绑定关系成功！");
        }
    }


    @AnonymousAccess
    @ApiOperation("知识概要展示")
    @PostMapping("selectByConditions")
    public ResultEntity<SysKnowledgeSummary> selectByConditions(@RequestParam String oneId,
                                                                @RequestParam String twoId,
                                                                @RequestParam String threeId) {
        SysKnowledgeSummaryExample example = new SysKnowledgeSummaryExample();
        SysKnowledgeSummaryExample.Criteria criteria = example.createCriteria();
        criteria.andOneTagIdEqualTo(oneId).andThreeTagIdEqualTo(threeId).andTwoTagIdEqualTo(twoId);
        List<SysKnowledgeSummary> sysKnowledgeSummaries = knowledgeSummaryMapper.selectByExampleWithBLOBs(example);
        if (sysKnowledgeSummaries != null && !sysKnowledgeSummaries.isEmpty()) {
            return new ResultEntity<SysKnowledgeSummary>(sysKnowledgeSummaries.get(0));
        } else return null;
    }


}
