package com.sztzjy.resource_center.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.SysResource;
import com.sztzjy.resource_center.entity.dto.SysResourceDto;
import com.sztzjy.resource_center.mapper.SysResourceMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.List;

/**
 * @Author xcj
 * @Date 2024/6/18
 */
@RestController
@Api(tags = "学习资源")
@RequestMapping("api/sys/learning/resource")
public class LearningResourceController {

    @Autowired
    private SysResourceMapper sysResourceMapper;
    @Value("${file.path}")
    private String filePath;



    @AnonymousAccess
    @ApiOperation("学习资源/资源列表展示/有重复数据")
    @PostMapping("selectResourceByRepeat")
    public ResultEntity<PageInfo<SysResourceDto>> selectResourceByRepeat(@ApiParam("课程ID") @RequestParam(required = false) String oneID,
                                                                         @ApiParam("章ID") @RequestParam(required = false) String twoID,
                                                                         @ApiParam("节ID") @RequestParam(required = false) String threeID,
                                                                         @ApiParam("资源名称") @RequestParam(required = false) String resourceName,
                                                                         @RequestParam Integer index,
                                                                         @RequestParam Integer size) {
        PageHelper.startPage(index, size);
        List<SysResourceDto> sysResourceDtos = sysResourceMapper.selectResourceByRepeat(oneID, twoID, threeID, resourceName);
        PageInfo<SysResourceDto> pageInfo = new PageInfo(sysResourceDtos);
        return new ResultEntity<PageInfo<SysResourceDto>>(pageInfo);
    }


    @AnonymousAccess
    @ApiOperation("学习资源/删除") //同步过来的不做删除
    @PostMapping("deleteResource")
    public ResultEntity<String> deleteResource(@ApiParam("来源是学校的不允许传过来") @RequestParam String id) {
        sysResourceMapper.deleteByPrimaryKey(id);
        return new ResultEntity<>(HttpStatus.OK, "删除成功!");
    }


    @GetMapping("/getResource")
    @ApiOperation("预览/获取视频或课件")
    @AnonymousAccess
    public ResponseEntity<FileSystemResource> streamVideo(@ApiParam("资源ID") @RequestParam String id,HttpServletResponse response) {

        SysResource sysResource = sysResourceMapper.selectByPrimaryKey(id);
        String url = sysResource.getUrl();
        String videoPath = filePath + url;
        File videoFile = new File(videoPath);
        if ("视频".equals(sysResource.getResourceType())) {
            if (videoFile.exists()) {
//                    Path path = Paths.get(videoPath);
                FileSystemResource fileSystemResource = new FileSystemResource(videoFile);
                return ResponseEntity.ok()
                        .contentType(MediaType.parseMediaType("video/mp4"))
                        .body(fileSystemResource);
            } else {
                return ResponseEntity.notFound().build();
            }
        } else if ("课件".equals(sysResource.getResourceType())) {
//            fileUtil.getPdf(response, name, url);
            if (videoFile.exists()) {
//                Path path = Paths.get(videoPath);
                FileSystemResource fileSystemResource = new FileSystemResource(videoFile);
                return ResponseEntity.ok()
                        .contentType(MediaType.parseMediaType("application/pdf"))
                        .body(fileSystemResource);
            } else {
                return ResponseEntity.notFound().build();
            }
        }
        return ResponseEntity.notFound().build();
    }
}
