package com.sztzjy.resource_center.controller;

import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.SysResourceData;
import com.sztzjy.resource_center.entity.SysResourceDataExample;
import com.sztzjy.resource_center.mapper.SysCaseQuestionMapper;
import com.sztzjy.resource_center.mapper.SysResourceDataMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import com.sztzjy.resource_center.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.UUID;

/**
 * @Author xcj
 * @Date 2024/7/15
 */
@Api(tags = "数据集相关")
@RestController
@RequestMapping("api/sys/resourceData")
public class ResourceDataController {
    @Autowired
    private SysResourceDataMapper sysResourceDataMapper;
    @Autowired
    IFileUtil fileUtil;

    /**
     * 数据集文件上传
     * 方法名：uploadResourceData
     * 参数：MultipartFile schoolId caseId
     * return: boolean
     */
    @AnonymousAccess
    @ApiOperation("数据集文件上传")
    @PostMapping("uploadResourceData")  //todo 管理员可以上传任意来源的案例题数据集
    private ResultEntity<String> uploadResourceData(@RequestParam MultipartFile file,
                                                    @RequestParam String schoolId,
                                                    @RequestParam String caseId) {
        try {
            String uploadUrl = fileUtil.upload(file);
            SysResourceData resourceData = new SysResourceData();
            resourceData.setResourceDataId(String.valueOf(UUID.randomUUID()));
            resourceData.setResourceName(file.getOriginalFilename());
            resourceData.setSize((double) file.getSize());
            resourceData.setSource(schoolId);
            resourceData.setCaseId(caseId);
            resourceData.setUrl(uploadUrl);
            sysResourceDataMapper.insert(resourceData);
            return new ResultEntity<>(HttpStatus.OK, "上传成功！");
        } catch (Exception e) {
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "上传失败！请联系管理员");
        }
    }

    /**
     * 数据集列表信息查询
     * 方法名：selectResourceDataList
     * 参数：caseId
     * return: List<SysResourceData>
     */
    @AnonymousAccess
    @ApiOperation("数据集列表信息查询")
    @PostMapping("selectResourceDataList")
    private ResultEntity<List<SysResourceData>> selectResourceDataList(@RequestParam String caseId) {
        SysResourceDataExample example = new SysResourceDataExample();
        example.createCriteria().andCaseIdEqualTo(caseId);
        return new ResultEntity<List<SysResourceData>>(sysResourceDataMapper.selectByExample(example));
    }


    /**
     * 数据集删除
     * 方法名：deleteResourceData
     * 参数：resourceId schoolId
     * return: boolean
     */
    @AnonymousAccess
    @ApiOperation("数据集删除")
    @PostMapping("deleteResourceData")
    private ResultEntity<String> deleteResourceData(@RequestParam String resourceId) {
        try {
            sysResourceDataMapper.deleteByPrimaryKey(resourceId);
        } catch (Exception e) {
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "删除失败！");
        }
        return new ResultEntity<>(HttpStatus.OK, "删除成功！");
    }


    /**
     * 数据集下载
     * 方法名：downloadResourceData
     * 参数：resourceId response
     * return: void
     */
    @AnonymousAccess
    @ApiOperation("数据集下载")
    @PostMapping("downloadResourceData")
    private Boolean selectResourceDataList(@RequestParam String resourceId,
                                           HttpServletResponse response) {
        try {
            SysResourceData sysResourceData = sysResourceDataMapper.selectByPrimaryKey(resourceId);
            String resourceName = sysResourceData.getResourceName();
            fileUtil.download(response, resourceName, sysResourceData.getUrl());
        } catch (Exception e) {
            return false;
        }
        return true;
    }
}
