package com.sztzjy.resource_center.controller;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.SysObjectiveQuestions;
import com.sztzjy.resource_center.entity.SysObjectiveQuestionsExample;
import com.sztzjy.resource_center.entity.SysTopicAndCourse;
import com.sztzjy.resource_center.entity.SysTopicAndCourseExample;
import com.sztzjy.resource_center.entity.dto.SysObjectiveQuestionsDto;
import com.sztzjy.resource_center.mapper.SysObjectiveQuestionsMapper;
import com.sztzjy.resource_center.mapper.SysTopicAndCourseMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import com.sztzjy.resource_center.util.file.IFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;

/**
 * @Author xcj
 * @Date 2024/6/18
 */
@Api(tags = "题库管理")
@RestController
@RequestMapping("api/sys/topic/resource")
public class TopicResourceController {

    @Autowired
    private SysObjectiveQuestionsMapper sysObjectiveQuestionMapper;
    @Autowired
    private SysTopicAndCourseMapper sysTopicAndCourseMapper;
    @Autowired
    IFileUtil fileUtil;
    @Value("${file.path}")
    private String filePath;


    @AnonymousAccess
    @ApiOperation("新增单个试题")  //新增 题干不允许重复
    @PostMapping("addTopic")
    private ResultEntity<String> addTopic(@RequestBody SysObjectiveQuestionsDto objectiveQuestion) {
        if (StringUtils.isBlank(objectiveQuestion.getContent())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "题干不能为空!");
        }
        if (StringUtils.isBlank(objectiveQuestion.getAnswer())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "答案不能为空!");
        }
        if (objectiveQuestion.getScore() == null) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "分数不能为空!");
        }
        if (StringUtils.isBlank(objectiveQuestion.getType())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "题型不能为空!");
        }
        if (StringUtils.isBlank(objectiveQuestion.getOneID())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "所属课程不能为空！");
        }
        if (objectiveQuestion.getType().equals("1") || objectiveQuestion.getType().equals("0")) {
            if (StringUtils.isBlank(objectiveQuestion.getQuestionA()) || StringUtils.isBlank(objectiveQuestion.getQuestionB())
                    || StringUtils.isBlank(objectiveQuestion.getQuestionC()) || StringUtils.isBlank(objectiveQuestion.getQuestionD()))
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "选择题至少有四个选项!");
        }
        if (objectiveQuestion.getType().equals("2")) {
            if (StringUtils.isBlank(objectiveQuestion.getQuestionA()) || StringUtils.isBlank(objectiveQuestion.getQuestionB()))
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "判断题必须有两个选项!");
        }
        SysObjectiveQuestionsExample example = new SysObjectiveQuestionsExample();
        example.createCriteria().andContentEqualTo(objectiveQuestion.getContent());
        List<SysObjectiveQuestions> sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByExample(example);
        if (!sysObjectiveQuestions.isEmpty()) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "题干不允许重复!");
        }
        //新增题库表
        String uuid = IdUtil.randomUUID();
        objectiveQuestion.setObjectiveId(uuid);
        objectiveQuestion.setCreateTime(new Date());
        objectiveQuestion.setIsDelete(false);
        sysObjectiveQuestionMapper.insert(objectiveQuestion);

        //新增题库关系表
        SysTopicAndCourse sysTopicAndCourse = new SysTopicAndCourse();
        sysTopicAndCourse.setId(IdUtil.randomUUID());
        sysTopicAndCourse.setTopicId(uuid);
        sysTopicAndCourse.setTopicType(objectiveQuestion.getType());
        sysTopicAndCourse.setOneId(objectiveQuestion.getOneID());
        sysTopicAndCourse.setOneName(objectiveQuestion.getOneName());
        if (StringUtils.isNotBlank(objectiveQuestion.getTwoID())) {
            sysTopicAndCourse.setTwoId(objectiveQuestion.getTwoID());
            sysTopicAndCourse.setTwoName(objectiveQuestion.getTwoName());
        }
        if (StringUtils.isNotBlank(objectiveQuestion.getThreeID())) {
            sysTopicAndCourse.setThreeId(objectiveQuestion.getThreeID());
            sysTopicAndCourse.setThreeName(objectiveQuestion.getThreeName());
        }
        sysTopicAndCourseMapper.insert(sysTopicAndCourse);
        return new ResultEntity<>(HttpStatus.OK, "新增题目成功！");
    }

    @AnonymousAccess
    @ApiOperation("本地excel导入试题")
    @PostMapping("importTopicByLocal")
    private ResultEntity<HttpStatus> importTopicByLocal(@RequestParam MultipartFile file,
                                                        @RequestParam String source) throws IOException, InvalidFormatException {
        if (file == null) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "导入文件不能为空");
        }
        return insertObjectiveByExcel(file, source);
    }


    @ApiOperation("资源中心批量导入")  //理解为批量添加已有资源的绑定关系
    @PostMapping("importTopic")
    private ResultEntity<String> importTopic(@RequestBody List<SysTopicAndCourse> list) {
        for (SysTopicAndCourse sysTopicAndCourse : list) {
            if (StringUtils.isBlank(sysTopicAndCourse.getOneId())
                    || StringUtils.isBlank(sysTopicAndCourse.getTwoId())
                    || StringUtils.isBlank(sysTopicAndCourse.getThreeId())
                    || StringUtils.isBlank(sysTopicAndCourse.getTopicId())) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "缺少必要的ID");
            } else if (StringUtils.isBlank(sysTopicAndCourse.getTopicType())) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "缺少题目类型");
            } else {
                sysTopicAndCourse.setId(IdUtil.randomUUID());
            }
        }
        for (SysTopicAndCourse sysTopicAndCourse : list) {
            SysTopicAndCourseExample example = new SysTopicAndCourseExample();
            example.createCriteria().andTopicIdEqualTo(sysTopicAndCourse.getTopicId())
                    .andOneIdEqualTo(sysTopicAndCourse.getOneId())
                    .andTwoIdEqualTo(sysTopicAndCourse.getTwoId())
                    .andThreeIdEqualTo(sysTopicAndCourse.getThreeId());
            List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);
            if (sysTopicAndCourses != null && !sysTopicAndCourses.isEmpty()) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "该章节下已存在此题目！");
            }
        }
        try {
            sysTopicAndCourseMapper.batchInsert(list);
        } catch (Exception e) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "导入失败，请联系管理员！");
        }
        return new ResultEntity<>(HttpStatus.OK, "添加成功！");
    }

    //客观题导入模板下载
    @GetMapping("downloadResource")
    @ApiOperation("客观题导入模板下载")
    @AnonymousAccess
    public void downloadResource(HttpServletResponse response) {
        fileUtil.download(response, "客观题导入模板.xlsx", filePath);
    }


    @AnonymousAccess
    @ApiOperation("条件查询/不重复")
    @PostMapping("selectTopicByConditions")  //todo 关于老师删除的题库如何查询
    private ResultEntity<PageInfo<SysObjectiveQuestionsDto>> selectTopicByConditions(@RequestParam Integer index,
                                                                                     @RequestParam Integer size,
                                                                                     @RequestParam(required = false) String oneID,
                                                                                     @RequestParam(required = false) String twoID,
                                                                                     @RequestParam(required = false) String threeID,
                                                                                     @RequestParam(required = false) String type,
                                                                                     @ApiParam("题干") @RequestParam(required = false) String content) {
        PageHelper.startPage(index, size);
        List<SysObjectiveQuestionsDto> list = sysObjectiveQuestionMapper.selectTopicByConditions(oneID, twoID, threeID, type, content);
        PageInfo<SysObjectiveQuestionsDto> pageInfo = new PageInfo(list);
        return new ResultEntity<PageInfo<SysObjectiveQuestionsDto>>(pageInfo);
    }


    @AnonymousAccess
    @ApiOperation("课程配置/题目条件查询")
    @PostMapping("getTopicByConfig")  //todo 关于老师删除的题库如何查询
    private ResultEntity<PageInfo<SysObjectiveQuestionsDto>> getTopicByConfig(@RequestParam Integer index,
                                                                              @RequestParam Integer size,
                                                                              @RequestParam(required = false) String oneID,
                                                                              @RequestParam(required = false) String twoID,
                                                                              @RequestParam(required = false) String threeID,
                                                                              @RequestParam(required = false) String type,
                                                                              @ApiParam("题干") @RequestParam(required = false) String content) {
        PageHelper.startPage(index, size);
        List<SysObjectiveQuestionsDto> list = sysObjectiveQuestionMapper.getTopicByConfig(oneID, twoID, threeID, type, null, content);
        PageInfo<SysObjectiveQuestionsDto> pageInfo = new PageInfo(list);
        return new ResultEntity<PageInfo<SysObjectiveQuestionsDto>>(pageInfo);
    }


    @AnonymousAccess
    @ApiOperation("客观题单个详情查看")
    @PostMapping("getById")
    private ResultEntity<SysObjectiveQuestions> getById(@RequestParam String id) {
        SysObjectiveQuestions sysObjectiveQuestions = sysObjectiveQuestionMapper.selectByPrimaryKey(id);
        return new ResultEntity<>(sysObjectiveQuestions);
    }


    //老师新增的题目正在使用的话资源中心只能查看，不能编辑和删除，
    // 如果老师自己删除了这个题目，资源中心题库管理也可以编辑和删除。如果题目正在被使用，则删除的时候提示：题目被xx课程xx章节使用，不能删除。
    @ApiOperation("编辑")
    @PostMapping("updateTopic")
    private ResultEntity<String> updateTopic(@ApiParam("修改绑定关系需要传topicAndCourseId") @RequestBody SysObjectiveQuestionsDto dto) {
        if (!dto.getSource().equals("管理员")) { //老师新增的不能编辑
            SysTopicAndCourseExample example = new SysTopicAndCourseExample();
            example.createCriteria().andTopicIdEqualTo(dto.getObjectiveId());
            List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);
            if (!sysTopicAndCourses.isEmpty()) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "无法删除，题目正在使用！");
            }
        }
        //ID不为空，修改绑定信息
        if (StringUtils.isNotBlank(dto.getTopicAndCourseId())) {
            SysTopicAndCourse sysTopicAndCourse = new SysTopicAndCourse();
            BeanUtils.copyProperties(dto, sysTopicAndCourse);
            sysTopicAndCourseMapper.updateByPrimaryKey(sysTopicAndCourse);
        }
        //修改题库基础信息
        SysObjectiveQuestions newData = new SysObjectiveQuestions();
        BeanUtils.copyProperties(dto, newData);
        sysObjectiveQuestionMapper.updateByPrimaryKey(newData);
        return new ResultEntity<>(HttpStatus.BAD_REQUEST, "编辑完成！");
    }

    @AnonymousAccess
    @ApiOperation("删除题目绑定关系")
    @PostMapping("deleteTopicBinding")
    private ResultEntity<String> deleteTopicBinding(@RequestParam String id,
                                                    @RequestParam(required = false) String oneId,
                                                    @RequestParam(required = false) String twoId,
                                                    @RequestParam(required = false) String threeId) {
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        SysTopicAndCourseExample.Criteria criteria = example.createCriteria();
        if (StringUtils.isNotBlank(oneId)) {
            criteria.andOneIdEqualTo(oneId);
        }
        if (StringUtils.isNotBlank(twoId)) {
            criteria.andTwoIdEqualTo(twoId);
        }
        if (StringUtils.isNotBlank(threeId)) {
            criteria.andThreeIdEqualTo(threeId);
        }
        criteria.andTopicIdEqualTo(id);
        sysTopicAndCourseMapper.deleteByExample(example);
        return new ResultEntity<>(HttpStatus.OK, "成功删除绑定关系！");
    }


    @AnonymousAccess
    @ApiOperation("批量删除题目绑定关系")
    @PostMapping("deleteTopicBindingByList")
    private ResultEntity<String> deleteTopicBindingByList(@RequestParam("传topicId和三级ID") @RequestBody List<SysTopicAndCourse> sysTopicAndCourses) {
        sysTopicAndCourseMapper.batchDelete(sysTopicAndCourses);
        return new ResultEntity<>(HttpStatus.OK, "成功删除绑定关系！");
    }


    //老师新增的题目正在使用的话资源中心只能查看，不能编辑和删除，
    // 如果老师自己删除了这个题目，资源中心题库管理也可以编辑和删除。如果题目正在被使用，则删除的时候提示：题目被xx课程xx章节使用，不能删除。
    @ApiOperation("删除")
    @PostMapping("deleteTopic")  //todo 接口暂时是给资源中心用，给老师删除用的话需要修改
    private ResultEntity<String> deleteTopic(@RequestParam String id,
                                             @ApiParam("来源") @RequestParam String inputType) {

        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andTopicIdEqualTo(id);
        List<SysTopicAndCourse> sysTopicAndCourses = sysTopicAndCourseMapper.selectByExample(example);
        if (!sysTopicAndCourses.isEmpty()) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "无法删除，题目正在使用！");
        }
        if (inputType.equals("1")) {  //老师新增，管理员无法删除
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "老师新增题目无法删除！");
        }
        sysObjectiveQuestionMapper.deleteByPrimaryKey(id);
        return new ResultEntity<>(HttpStatus.OK, "成功删除！");
    }


    //如果是内置增加 则需要提供课程和章节信息
    //如果是老师增加 则只需要提供课程信息
    public ResultEntity<HttpStatus> insertObjectiveByExcel(MultipartFile file, String source) throws IOException, InvalidFormatException {
        List<SysObjectiveQuestions> objectiveQuestionList = new ArrayList<>();
        Workbook workbook = WorkbookFactory.create(file.getInputStream());
        Sheet sheet = workbook.getSheetAt(0);

        // 迭代每一行
        Iterator<Row> iterator = sheet.iterator();
        iterator.next();
        while (iterator.hasNext()) {
            Row row = iterator.next();

            /*解析列，下标从0开始*/
            Cell cell0 = row.getCell(0);
            Cell cell1 = row.getCell(1);
            Cell cell2 = row.getCell(2);
            Cell cell3 = row.getCell(3);
            Cell cell4 = row.getCell(4);
            Cell cell5 = row.getCell(5);
            Cell cell6 = row.getCell(6);
            Cell cell7 = row.getCell(7);
            Cell cell8 = row.getCell(8);
            Cell cell9 = row.getCell(9);
            //判断题的BCDE选项和解析可能为空
            if (cell0 == null || cell1 == null || cell2 == null || cell6 == null || cell7 == null || cell9 == null) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "必填项不能为空");
            }

            String content = this.getCellValue(cell0);//题干
            String a = this.getCellValue(cell1); //选项A
            String b = this.getCellValue(cell2);//选项B
            String type = this.getCellValue(cell6);//题型
            String answer = this.getCellValue(cell7);//答案
            String score = this.getCellValue(cell9);//分值
            String c = cell3 != null ? this.getCellValue(cell3) : null;
            String d = cell4 != null ? this.getCellValue(cell4) : null;
            String e = cell5 != null ? this.getCellValue(cell5) : null;
            String analyze = cell8 != null ? this.getCellValue(cell8) : null;


            SysObjectiveQuestions obj = new SysObjectiveQuestions();
            obj.setObjectiveId(String.valueOf(UUID.randomUUID()));
            obj.setContent(content);
            obj.setType(type);
            obj.setSource(source);
            obj.setQuestionA(a);
            obj.setQuestionB(b);
            obj.setQuestionC(c);
            obj.setQuestionD(d);
            obj.setQuestionE(e);
            obj.setAnswer(answer);
            obj.setAnalysis(analyze);
            obj.setScore(BigDecimal.valueOf(Integer.parseInt(score)));
            obj.setCreateTime(new Date());
            obj.setIsDelete(false);
            objectiveQuestionList.add(obj);
        }
        int result = sysObjectiveQuestionMapper.insertBatch(objectiveQuestionList);
        if (result > 0) {
            return new ResultEntity<>(HttpStatus.OK, "批量导入成功！");
        } else {
            return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "批量导入失败，请联系管理员！");
        }
    }


    public String getCellValue(Cell cell) {
        if (cell.getCellType() == Cell.CELL_TYPE_BOOLEAN) {
            return String.valueOf(cell.getBooleanCellValue());
        } else if (cell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
            Double d = cell.getNumericCellValue();
            return String.valueOf(d.intValue());
        }
        return String.valueOf(cell.getStringCellValue());
    }
}
