package com.sztzjy.resource_center.controller.admin;

import cn.hutool.core.util.IdUtil;
import com.sztzjy.resource_center.entity.admin.AdminCaseDto;
import com.sztzjy.resource_center.entity.admin.AdminCaseWithBLOBs;
import com.sztzjy.resource_center.entity.admin.AdminFile;
import com.sztzjy.resource_center.entity.admin.AdminLabel;
import com.sztzjy.resource_center.mapper.admin.AdminCaseMapper;
import com.sztzjy.resource_center.mapper.admin.AdminFileMapper;
import com.sztzjy.resource_center.mapper.admin.AdminLabelMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import com.sztzjy.resource_center.util.file.IFileUtil;
import io.swagger.annotations.Api;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author xcj
 * @Date 2024/7/11
 */
@RestController
@Api(tags = "管理员案例集锦页面")
@RequestMapping("api/admin/case")
public class AdminCaseController {
    @Autowired
    private AdminCaseMapper adminCaseMapper;
    @Autowired
    private AdminLabelMapper adminLabelMapper;
    @Autowired
    private AdminFileMapper adminFileMapper;
    @Autowired
    private IFileUtil fileUtil;

    @PostMapping("新增")
    @Transactional
    ResultEntity<String> add(@ModelAttribute AdminCaseDto adminCaseDto) {
        if (StringUtils.isBlank(adminCaseDto.getName())) {
            return new ResultEntity<>(HttpStatus.OK, "请输入名称！");
        }
        if (adminCaseDto.getAdminLabelList() == null || adminCaseDto.getAdminLabelList().isEmpty()) {
            return new ResultEntity<>(HttpStatus.OK, "至少选择一个标签！");
        }
        List<AdminLabel> adminLabelList = adminCaseDto.getAdminLabelList();
        for (AdminLabel adminLabel : adminLabelList) {
            if (StringUtils.isBlank(adminLabel.getLabelId()) || StringUtils.isBlank(adminLabel.getName()) || StringUtils.isBlank(adminLabel.getType())) {
                return new ResultEntity<>(HttpStatus.OK, "标签数据不完整！");
            }
        }
        if (StringUtils.isBlank(adminCaseDto.getCaseContent()) && adminCaseDto.getCaseContentFile() == null) {
            return new ResultEntity<>(HttpStatus.OK, "请输入内容或者上传文件！");
        }

        //开始新增
        AdminCaseWithBLOBs adminCase = new AdminCaseWithBLOBs();
        String caseId = IdUtil.randomUUID();
        adminCase.setId(caseId);
        adminCase.setName(adminCaseDto.getName());
        adminCase.setStatus(0); //新增默认下架，发布后上架
        if (adminCaseDto.getPictureFile() != null) {
            String pictureUrl = fileUtil.upload(adminCaseDto.getPictureFile());
            adminCase.setPictureUrl(pictureUrl);
        }

        //设置内容
        if (adminCaseDto.getCaseContentFile() != null) {
            String contentUrl = fileUtil.upload(adminCaseDto.getCaseContentFile());
            adminCase.setContent(contentUrl);  //传的文件就存地址
        }
        if (StringUtils.isNotBlank(adminCaseDto.getCaseContent())) {
            adminCase.setContent(adminCaseDto.getCaseContent());
        }

        //设置目录
        if (adminCaseDto.getCaseCatalogFile() != null) {
            String caseCatalogUrl = fileUtil.upload(adminCaseDto.getCaseCatalogFile());
            adminCase.setCatalog(caseCatalogUrl);  //传的文件就存地址
        }
        if (StringUtils.isNotBlank(adminCaseDto.getCaseCatalog())) {
            adminCase.setCatalog(adminCaseDto.getCaseCatalog());
        }

        //设置数据文件
        if (adminCaseDto.getDataFile() != null && !adminCaseDto.getDataFile().isEmpty()) {
            List<MultipartFile> dataFileList = adminCaseDto.getDataFile();
            List<AdminFile> list = new ArrayList<>();
            for (MultipartFile file : dataFileList) {
                AdminFile adminFile = new AdminFile();
                adminFile.setFileId(IdUtil.randomUUID());
                String pictureUrl = fileUtil.upload(file);
                String name = file.getName();
                adminFile.setFileUrl(pictureUrl);
                adminFile.setName(name);
                adminFile.setSource("案例表");
                adminFile.setDataCaseId(caseId);
                list.add(adminFile);
            }
            //批量新增数据文件
            adminFileMapper.insertBatch(list);
        }

        //批量新增标签
        adminLabelMapper.insertBatch(adminLabelList);
        //新增案例
        adminCaseMapper.insert(adminCase);
        return new ResultEntity<>(HttpStatus.OK, "新增成功！");
    }


}
