package com.sztzjy.resource_center.controller.admin;

import cn.hutool.core.util.IdUtil;
import com.sztzjy.resource_center.entity.admin.*;
import com.sztzjy.resource_center.mapper.admin.AdminDataMapper;
import com.sztzjy.resource_center.mapper.admin.AdminFileMapper;
import com.sztzjy.resource_center.mapper.admin.AdminLabelMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import com.sztzjy.resource_center.util.file.IFileUtil;
import io.swagger.annotations.Api;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author xcj
 * @Date 2024/7/11
 */
@RestController
@Api(tags = "管理员数据集锦页面")
@RequestMapping("api/admin/data")
public class AdminDataController {
    @Autowired
    private IFileUtil fileUtil;
    @Autowired
    private AdminLabelMapper adminLabelMapper;
    @Autowired
    private AdminFileMapper adminFileMapper;
    @Autowired
    private AdminDataMapper adminDataMapper;

    @PostMapping("新增")
    @Transactional
    ResultEntity<String> add(@ModelAttribute AdminDataDto adminDataDto) {
        if (StringUtils.isBlank(adminDataDto.getName())) {
            return new ResultEntity<>(HttpStatus.OK, "请输入名称！");
        }
        if (StringUtils.isBlank(adminDataDto.getFieldDescription())) {
            return new ResultEntity<>(HttpStatus.OK, "请输入字段描述！");
        }
        if (StringUtils.isBlank(adminDataDto.getDataScenarios())) {
            return new ResultEntity<>(HttpStatus.OK, "请输入数据应用场景！");
        }
        if (adminDataDto.getAdminLabelList() == null || adminDataDto.getAdminLabelList().isEmpty()) {
            return new ResultEntity<>(HttpStatus.OK, "至少选择一个标签！");
        }

        List<AdminLabel> adminLabelList = adminDataDto.getAdminLabelList();

        for (AdminLabel adminLabel : adminLabelList) {
            if (StringUtils.isBlank(adminLabel.getLabelId()) || StringUtils.isBlank(adminLabel.getName()) || StringUtils.isBlank(adminLabel.getType())) {
                return new ResultEntity<>(HttpStatus.OK, "标签数据不完整！");
            }
        }

        //新增
        AdminDataWithBLOBs adminDataWithBLOBs =new AdminDataWithBLOBs();
        String dataId = IdUtil.randomUUID();
        adminDataWithBLOBs.setDataId(dataId);
        adminDataWithBLOBs.setName(adminDataDto.getName());
        adminDataWithBLOBs.setDataScenarios(adminDataDto.getDataScenarios());
        adminDataWithBLOBs.setFieldDescription(adminDataDto.getFieldDescription());
        if (adminDataDto.getPictureUrl() != null) {
            String pictureUrl = fileUtil.upload(adminDataDto.getPictureFile());
            adminDataWithBLOBs.setPictureUrl(pictureUrl);
        }

        //设置数据文件
        if (adminDataDto.getDataFile() != null && !adminDataDto.getDataFile().isEmpty()) {
            List<MultipartFile> dataFileList = adminDataDto.getDataFile();
            List<AdminFile> list = new ArrayList<>();
            for (MultipartFile file : dataFileList) {
                AdminFile adminFile = new AdminFile();
                adminFile.setFileId(IdUtil.randomUUID());
                String pictureUrl = fileUtil.upload(file);
                String name = file.getName();
                adminFile.setFileUrl(pictureUrl);
                adminFile.setName(name);
                adminFile.setSource("数据表");
                adminFile.setDataCaseId(dataId);
                list.add(adminFile);
            }
            //批量新增数据文件
            adminFileMapper.insertBatch(list);
        }

        //批量新增标签
        adminLabelMapper.insertBatch(adminLabelList);
        //新增案例
        adminDataMapper.insert(adminDataWithBLOBs);
        return new ResultEntity<>(HttpStatus.OK,"新增成功！");
    }
}
