package com.sztzjy.resource_center.controller.admin;

import cn.hutool.core.util.IdUtil;
import com.sztzjy.resource_center.entity.admin.AdminLabel;
import com.sztzjy.resource_center.entity.admin.AdminLabelExample;
import com.sztzjy.resource_center.mapper.admin.AdminLabelMapper;
import com.sztzjy.resource_center.util.ResultEntity;
import io.swagger.annotations.Api;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author xcj
 * @Date 2024/7/11
 */
@RestController
@Api(tags = "管理员标签管理")
@RequestMapping("api/admin/label")
public class AdminLabelController {
    @Autowired
    private AdminLabelMapper adminLabelMapper;


    @PostMapping("新增")
    private ResultEntity<String> addLabel(@RequestBody AdminLabel adminLabel) {
        if (StringUtils.isBlank(adminLabel.getName())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "标签名称不能为空！");
        }
        if (StringUtils.isBlank(adminLabel.getType())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "标签类型不能为空！");
        }
        adminLabel.setLabelId(IdUtil.randomUUID());
        adminLabelMapper.insert(adminLabel);
        return new ResultEntity<>(HttpStatus.OK, "新增成功！");
    }

    @PostMapping("修改")
    private ResultEntity<String> updateLabel(@RequestBody AdminLabel adminLabel) {
        if (StringUtils.isBlank(adminLabel.getLabelId())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "缺少必要的ID！");
        }
        if (StringUtils.isBlank(adminLabel.getName())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "标签名称不能为空！");
        }
        if (StringUtils.isBlank(adminLabel.getType())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "标签类型不能为空！");
        }
        AdminLabelExample example = new AdminLabelExample();
        example.createCriteria().andTypeEqualTo(adminLabel.getType()).andNameEqualTo(adminLabel.getName());
        List<AdminLabel> adminLabels = adminLabelMapper.selectByExample(example);
        if (adminLabels != null && !adminLabels.isEmpty()) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "标签名称不能重复！");
        } else {
            adminLabelMapper.updateByPrimaryKey(adminLabel);
            return new ResultEntity<>(HttpStatus.OK, "修改成功！");
        }
    }

    @PostMapping("删除")
    private ResultEntity<String> deleteLabel(@RequestParam String id) {
        adminLabelMapper.deleteByPrimaryKey(id);
        return new ResultEntity<>(HttpStatus.OK, "删除成功！");
    }

    @PostMapping("展示")
    private ResultEntity<List<AdminLabel>> getLabel() {
        AdminLabelExample example = new AdminLabelExample();
        List<AdminLabel> adminLabels = adminLabelMapper.selectByExample(example);
        return new ResultEntity<>(adminLabels);
    }

    @PostMapping("下拉框")
    private ResultEntity<List<AdminLabel>> getLabelByType(@RequestParam String type) {
        AdminLabelExample example = new AdminLabelExample();
        example.createCriteria().andTypeEqualTo(type);
        List<AdminLabel> adminLabels = adminLabelMapper.selectByExample(example);
        return new ResultEntity<>(adminLabels);
    }
}
