package com.sztzjy.resource_center.controller.api;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.entity.*;
import com.sztzjy.resource_center.mapper.SysCaseQuestionMapper;
import com.sztzjy.resource_center.mapper.SysCaseQuestionStepMapper;
import com.sztzjy.resource_center.mapper.SysOneCatalogMapper;
import com.sztzjy.resource_center.mapper.SysTopicAndCourseMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.UUID;

@RestController
@Api(tags = "课程方面API")
@RequestMapping("api/tea/CaseApi")
public class CaseApi {

    @Autowired
    private SysCaseQuestionStepMapper caseQuestionStepMapper;
    @Autowired
    private SysCaseQuestionMapper caseQuestionMapper;
    @Autowired
    private SysTopicAndCourseMapper topicAndCourseMapper;
    @Autowired
    private SysOneCatalogMapper sysOneCatalogMapper;

    //案例题新增 SysCaseQuestion
    @AnonymousAccess
    @ApiOperation("案例题新增")
    @PostMapping("insertCase")
    public Boolean insertCase(@ApiParam("目前二级三级ID可以为空，来源和题型要传,上架状态传true") @RequestBody SysCaseQuestion sysCaseQuestion) {
        if (("3").equals(sysCaseQuestion.getType())) {
            return false;
        }
        if (StringUtils.isBlank(sysCaseQuestion.getOneName())) {  //todo 目前二级三级ID可以为空
            return false;
        }
        if (StringUtils.isBlank(sysCaseQuestion.getContent()) || StringUtils.isBlank(sysCaseQuestion.getTitle())) {
            return false;
        }
        if (StringUtils.isBlank(sysCaseQuestion.getSource())) {
            return false;
        }
        String title = sysCaseQuestion.getTitle();
        SysCaseQuestionExample example = new SysCaseQuestionExample();
        example.createCriteria().andTitleEqualTo(title);
        List<SysCaseQuestion> sysCaseQuestions = caseQuestionMapper.selectByExample(example);
        if (!sysCaseQuestions.isEmpty()) {
            return false;
        }

        String uuid = IdUtil.randomUUID();
        sysCaseQuestion.setCaseId(uuid);
        sysCaseQuestion.setCreateTime(new Date());
        sysCaseQuestion.setUnmountStatus(false);
        caseQuestionMapper.insert(sysCaseQuestion);
        //新增绑定关系
        SysTopicAndCourse sysTopicAndCourse = new SysTopicAndCourse();
        sysTopicAndCourse.setId(IdUtil.randomUUID());
        sysTopicAndCourse.setTopicType("1");
        sysTopicAndCourse.setTopicId(uuid);
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(sysCaseQuestion.getOneName());
        String oneId = sysOneCatalogs.getOneId();
        sysTopicAndCourse.setOneId(oneId);
        sysTopicAndCourse.setOneName(sysCaseQuestion.getOneName());
        if (StringUtils.isNotBlank(sysCaseQuestion.getTwoId())) {
            sysTopicAndCourse.setTwoId(sysCaseQuestion.getTwoId());
        }
        if (StringUtils.isNotBlank(sysCaseQuestion.getThreeId())) {
            sysTopicAndCourse.setThreeId(sysCaseQuestion.getThreeId());
        }
        topicAndCourseMapper.insert(sysTopicAndCourse);
        return true;
    }

    private SysOneCatalog getSysOneCatalogs(String systemOwner) {
        SysOneCatalogExample example = new SysOneCatalogExample();
        example.createCriteria().andOneNameEqualTo(systemOwner);
        List<SysOneCatalog> sysOneCatalogs = sysOneCatalogMapper.selectByExample(example);
        return sysOneCatalogs.get(0);
    }

    //案例题列表查询 schoolId keyword index size systemOwner
    @AnonymousAccess
    @ApiOperation("案例题页面查询")
    @PostMapping("selectCaseList")
    public PageInfo<SysCaseQuestion> selectCaseByConditions(@RequestParam Integer index,
                                                            @RequestParam Integer size,
                                                            @RequestParam(required = false) String keyword,
                                                            @RequestParam(required = false) String systemOwner,
                                                            @RequestParam(required = false) String schoolId) {
        PageHelper.startPage(index, size);
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(systemOwner);
        String oneId = sysOneCatalogs.getOneId();
        List<SysCaseQuestion> list = caseQuestionMapper.selectCaseByConditions(keyword, oneId, schoolId);
        PageInfo<SysCaseQuestion> pageInfo = new PageInfo<SysCaseQuestion>(list);
        return pageInfo;
    }

    //案例题详情查询 caseId
    @AnonymousAccess
    @ApiOperation("案例题详细内容展示")
    @PostMapping("selectCaseDetails")
    public SysCaseQuestion selectCaseByConditionsByID(@RequestParam String caseId) {
        return caseQuestionMapper.selectByPrimaryKey(caseId);

    }


    //案例题删除 caseId
    @AnonymousAccess
    @ApiOperation("案例题删除") //逻辑删除
    @PostMapping("caseDelete")
    public Boolean deleteCase(@RequestParam String caseId) {
        SysCaseQuestion sysCaseQuestion = caseQuestionMapper.selectByPrimaryKey(caseId);
        String source = sysCaseQuestion.getSource();
        //todo 管理员任意删除 老师只能删除自己上传的
        if (!"管理员".equals(source)) {
            SysCaseQuestion dataSysCaseQuestion = caseQuestionMapper.selectByPrimaryKey(sysCaseQuestion.getCaseId());
            dataSysCaseQuestion.setUnmountStatus(true);
            caseQuestionMapper.updateByPrimaryKey(dataSysCaseQuestion);
            return true;
        } else {
            return false;
        }
    }


    private List<SysTopicAndCourse> getSysTopicAndCourses(String caseId) {
        SysTopicAndCourseExample example = new SysTopicAndCourseExample();
        example.createCriteria().andTopicIdEqualTo(caseId).andTopicTypeEqualTo("1");
        List<SysTopicAndCourse> sysTopicAndCourses = topicAndCourseMapper.selectByExample(example);
        return sysTopicAndCourses;
    }


    //案例题编辑 SysCaseQuestion
    @AnonymousAccess
    @ApiOperation("案例题编辑")
    @PostMapping("updateCase")
    public Boolean updateCase(@RequestBody SysCaseQuestion sysCaseQuestion) {
        List<SysTopicAndCourse> sysTopicAndCourses = getSysTopicAndCourses(sysCaseQuestion.getCaseId());
        if (!sysTopicAndCourses.isEmpty()) {
            return false;
        } else {
            caseQuestionMapper.updateByPrimaryKey(sysCaseQuestion);
            return true;
        }
    }


    //案例题步骤信息查询 caseId
    @AnonymousAccess
    @ApiOperation("案例步骤展示")
    @PostMapping("selectCaseStepDetailByCaseId")
    public List<SysCaseQuestionStepWithBLOBs> selectCaseStep(@ApiParam("案例题ID") @RequestParam String caseId) {
        SysCaseQuestionStepExample example = new SysCaseQuestionStepExample();
        example.createCriteria().andCaseIdEqualTo(caseId);
        example.setOrderByClause("sort ASC");
        return caseQuestionStepMapper.selectByExampleWithBLOBs(example);
    }


    //案例题步骤新增 SysCaseQuestionStep
    @AnonymousAccess
    @ApiOperation("案例题添加步骤")
    @PostMapping("insertCaseStep")
    public Boolean insertCaseStepByCaseId(@RequestBody SysCaseQuestionStepWithBLOBs caseQuestionStep) {
        if (caseQuestionStep.getCaseId().isEmpty()) {
            return false;
        }
        if (caseQuestionStep.getTitle().isEmpty()
                || caseQuestionStep.getQuestion().isEmpty()
                || caseQuestionStep.getAnswer().isEmpty()
                || caseQuestionStep.getContent().isEmpty()) {
            return false;
        }
        try {
            caseQuestionStep.setCaseStepId(String.valueOf(UUID.randomUUID()));
            caseQuestionStepMapper.insert(caseQuestionStep);
            return true;
        } catch (Exception e) {
            return false;
        }
    }


    //案例步骤编辑 SysCaseQuestionStepWithBLOBs
    @AnonymousAccess
    @ApiOperation("案例题步骤编辑")
    @PostMapping("updateCaseStep")
    public Boolean updateCaseStep(@RequestBody SysCaseQuestionStepWithBLOBs caseQuestionStepWithBLOBs) {
        List<SysTopicAndCourse> sysTopicAndCourses = getSysTopicAndCourses(caseQuestionStepWithBLOBs.getCaseId());
        if (!sysTopicAndCourses.isEmpty()) {
            return false;
        } else {
            caseQuestionStepMapper.updateByPrimaryKey(caseQuestionStepWithBLOBs);
            return true;
        }
    }

    //案例步骤删除 caseStepId
    @AnonymousAccess
    @ApiOperation("案例题步骤删除")
    @PostMapping("deleteCaseStep")
    public Boolean deleteCaseStep(@RequestParam String caseStepId) {
        List<SysTopicAndCourse> sysTopicAndCourses = getSysTopicAndCourses(caseStepId);
        if (!sysTopicAndCourses.isEmpty()) {
            return false;
        } else {
            caseQuestionStepMapper.deleteByPrimaryKey(caseStepId);
            return true;
        }
    }

    //案例题步骤内容查看 caseStepId
    @PostMapping("selectCaseStepDetails")
    @ApiOperation("案例题步骤内容查看")
    public SysCaseQuestionStep selectCaseStepDetails(@ApiParam("案例题步骤ID") @RequestParam String caseStepId) {
        return caseQuestionStepMapper.selectByPrimaryKey(caseStepId);
    }

}
