package com.sztzjy.resource_center.controller.api;

import cn.hutool.core.util.IdUtil;
import com.sztzjy.resource_center.annotation.AnonymousAccess;
import com.sztzjy.resource_center.config.Constant;
import com.sztzjy.resource_center.entity.*;
import com.sztzjy.resource_center.mapper.SysOneCatalogMapper;
import com.sztzjy.resource_center.mapper.SysThreeCatalogMapper;
import com.sztzjy.resource_center.mapper.SysTwoCatalogMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@RestController
@Api(tags = "课程方面API")
@RequestMapping("api/sys/courseApi")
public class CourseApi {
    @Autowired
    SysTwoCatalogMapper twoCatalogMapper;
    @Autowired
    SysOneCatalogMapper oneCatalogMapper;
    @Autowired
    SysThreeCatalogMapper threeCatalogMapper;

    @AnonymousAccess
    @ApiOperation("查看二级目录")
    @PostMapping("getAllTwoCatalogList")
    public List<SysTwoCatalog> getAllTwoCatalogList(@RequestParam String schoolId, @RequestParam String systemOwner) {
        SysOneCatalogExample sysOneCatalogExample = new SysOneCatalogExample();
        sysOneCatalogExample.createCriteria().andOneNameEqualTo(systemOwner);
        SysOneCatalog sysOneCatalog = oneCatalogMapper.selectByExample(sysOneCatalogExample).get(0);
        SysTwoCatalogExample example = new SysTwoCatalogExample();
        SysTwoCatalogExample.Criteria criteria = example.createCriteria();
        ArrayList<String> list = new ArrayList<>();
        list.add(schoolId);
        list.add(Constant.ADMIN);
        criteria.andCreatorIn(list);
        criteria.andOneIdEqualTo(sysOneCatalog.getOneId());
        example.setOrderByClause("sort ASC");
        List<SysTwoCatalog> sysTwoCatalogs = twoCatalogMapper.selectByExample(example);
        return sysTwoCatalogs;
    }

    @PostMapping("selectThreeCatalogListByTwoId")
    @ApiOperation("根据二级目录ID查寻所有三级目录信息")
    @AnonymousAccess
    public List<SysThreeCatalog> selectChapterByCourseId(@RequestParam String twoId,@RequestParam String schoolId) {
        SysThreeCatalogExample example = new SysThreeCatalogExample();
        ArrayList<String> list = new ArrayList<>();
        list.add(schoolId);
        list.add(Constant.ADMIN);
        example.createCriteria().andTwoIdEqualTo(twoId).andCreatorIn(list);
        example.setOrderByClause("sort asc");
        List<SysThreeCatalog> sysThreeCatalogs = threeCatalogMapper.selectByExample(example);
        return sysThreeCatalogs;
    }

    //增加二级目录 SysTwoCatalog systemOwner
    @AnonymousAccess
    @ApiOperation("添加二级目录")
    @PostMapping("insertSysTwoCatalog")
    public Boolean insertSysTwoCatalog(@RequestBody SysTwoCatalog sysTwoCatalog) {
        SysOneCatalog sysOneCatalogs = getSysOneCatalogs(sysTwoCatalog.getOneId());   //一级目录name放ID里
        if (sysOneCatalogs == null) {
            return false;
        }
        String oneId = sysOneCatalogs.getOneId();
        sysTwoCatalog.setTwoId(IdUtil.randomUUID());
        sysTwoCatalog.setOneId(oneId);
        sysTwoCatalog.setCreateTime(new Date());
        sysTwoCatalog.setSort(null);
        twoCatalogMapper.insert(sysTwoCatalog);
        return true;
    }

    private SysOneCatalog getSysOneCatalogs(String systemOwner) {
        SysOneCatalogExample example = new SysOneCatalogExample();
        example.createCriteria().andOneNameEqualTo(systemOwner);
        List<SysOneCatalog> sysOneCatalogs = oneCatalogMapper.selectByExample(example);
        return sysOneCatalogs.get(0);
    }

    //增加三级目录 SysThreeCatalog schoolId systemOwner
    @AnonymousAccess
    @ApiOperation("添加三级目录")
    @PostMapping("insertSysThreeCatalog")
    public Boolean insertSysThreeCatalog(@RequestBody SysThreeCatalog sysThreeCatalog) {
        try {
            SysOneCatalog sysOneCatalogs = getSysOneCatalogs(sysThreeCatalog.getOntId()); //一级目录name放ID里
            sysThreeCatalog.setThreeId(IdUtil.randomUUID());
            sysThreeCatalog.setOntId(sysOneCatalogs.getOneId());
            sysThreeCatalog.setCreateTime(new Date());
            threeCatalogMapper.insert(sysThreeCatalog);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    //删除二级目录 TwoId
    @AnonymousAccess
    @ApiOperation("删除二级目录")
    @PostMapping("deleteTwoCatalog")
    public Boolean deleteTwoCatalog(@RequestParam String twoId) {
        try {
            SysTwoCatalog sysTwoCatalog = twoCatalogMapper.selectByPrimaryKey(twoId);
            if (sysTwoCatalog.getCreator().equals("管理员")) {
                return false;
            }
            twoCatalogMapper.deleteByPrimaryKey(twoId);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }


    //删除三级目录 ThreeId
    @AnonymousAccess
    @ApiOperation("删除三级目录")
    @PostMapping("deleteThreeCatalog")
    public Boolean deleteThreeCatalog(@RequestParam String threeId) {
        try {
            SysThreeCatalog sysThreeCatalog = threeCatalogMapper.selectByPrimaryKey(threeId);
            if (sysThreeCatalog.getCreator().equals("管理员")) {
                return false;
            }
            threeCatalogMapper.deleteByPrimaryKey(threeId);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

}
